
import 'package:hub/shared/components/molecules/modules/index.dart';
import 'package:hub/shared/helpers/database/database_helper.dart';
import 'package:hub/shared/helpers/storage/base_storage.dart';
import 'package:hub/shared/helpers/storage/storage_helper.dart';
import 'package:hub/shared/extensions/string_extensions.dart';
import 'package:sqflite/sqflite.dart';

abstract class LicenseLocalDataSource {
  Future<void> init();
  Future<void> setDisplayByKey(final List<String> key, final String display);
  Future<bool> isNewVersion();
  Future<String?> get(String key);
  Future<void> set<T>(String key, T value);
  Future<void> del(String key);
  Future<void> clear();
}

class LicenseLocalDataSourceImpl implements LicenseLocalDataSource {
  static final LicenseLocalDataSourceImpl _instance = LicenseLocalDataSourceImpl._internal();
  factory LicenseLocalDataSourceImpl() => _instance;
  LicenseLocalDataSourceImpl._internal();


  @override
  Future<void> init() async {
    await DatabaseStorage.instance.init();
  }

  @override
  Future<void> setDisplayByKey(final List<String?> key, final String display) async {
    if (key.isEmpty) return;
    for (var element in key) {
      if (element != null) {
        await set(element, {
          'display': display,
          'expirationDate': '',
          'startDate': '',
          'quantity': '',
        });
      }
    }
  }

  @override
  Future<bool> isNewVersion() async {
    final String? reponse = await StorageHelper().get(KeychainStorageKey.isNewVersion.value);
    final bool isNewVersion = reponse.toBoolean();
    return isNewVersion;
  }

  @override
  Future<String?> get(String key) async {
    var response = await DatabaseStorage.database.query(tableLicense, where: 'key = ?', whereArgs: [key]);
    if (response.isEmpty) {
      return null;
    }
    return response.first.toString();
  }

  @override
  Future<void> set<T>(String key, T value) async {
    value as Map<String, dynamic>;

    await DatabaseStorage.database.insert(
        tableLicense,
        {
          'key': key,
          'display': value['display'],
          'expirationDate': value['expirationDate'],
          'startDate': value['startDate'],
          'quantity': value['quantity'],
        },
        conflictAlgorithm: ConflictAlgorithm.replace);
    return;
  }

  @override
  Future<void> del(String key) async {
    await DatabaseStorage.database.delete(tableLicense, where: 'key = ?', whereArgs: [key]);
  }

  @override
  Future<void> clear() async {
    await DatabaseStorage.database.delete(tableLicense);
  }
}
